@@ -1,4 +1,4 @@
-# Copyright (c) 2013, Red Hat, Inc. All rights reserved.
+# Copyright (C) 2013-2014 Red Hat, Inc. All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
 # modification, are permitted provided that the following conditions are met:
@@ -31,13 +31,9 @@
 System software management.
 
 Usage:
-    %(cmd)s list pkgs
-        [(--available | --all) [--repoid <repository>]]
-        [--allow-duplicates] [<package> ...]
-    %(cmd)s list repos [--disabled | --all]
-    %(cmd)s list files [-t <file_type>] <package>
-    %(cmd)s show pkg [(--repoid <repository>) | --installed] <package>
-    %(cmd)s show repo <repository>
+    %(cmd)s search [(--repoid <repository>)] [--allow-duplicates] <package>...
+    %(cmd)s list (--help | <what> [<args>...])
+    %(cmd)s show (--help | <what> [<args>...])
     %(cmd)s install [--force] [--repoid <repository>] <package> ...
     %(cmd)s install --uri <uri>
     %(cmd)s update [--force] [--repoid <repository>] <package> ...
@@ -81,9 +77,8 @@ Options:
                    searched for.
     --uri <uri>    Operate upon an rpm package available on remote system
                    through http or ftp service.
-    -t --type (file | directory | device | symlink | fifo)
-                   List only particular file type.
     --installed    Limit the query only on installed packages.
+    --help         Get a detailed help for subcommand.
 
 Specifying <package>:
     Package can be given in one of following notations:
@@ -97,120 +92,30 @@ Specifying <package>:
     Bottom most notations allow to precisely identify particular package.
 """
 
-import itertools
-
 from lmi.scripts import software
 from lmi.scripts.common import command
 from lmi.scripts.common import errors
 from lmi.scripts.common import get_logger
 from lmi.scripts.common.formatter import command as fcmd
+from lmi.scripts.software.cmd_list import Lister
 
 LOG = get_logger(__name__)
 
-class PkgLister(command.LmiInstanceLister):
-    DYNAMIC_PROPERTIES = True
-    ARG_ARRAY_SUFFIX = '_array'
+class Search(command.LmiInstanceLister):
+    ARRAY_SUFFIX = '_array'
+    PROPERTIES = (
+            ('NEVRA', 'ElementName'),
+            ('Installed', lambda i: i.InstallDate is not None),
+            ('Summary', 'Caption'))
 
-    def execute(self, ns,
-            _available=False,
-            _all=False,
-            _repoid=None,
-            _allow_duplicates=False,
-            package_array=None):
-        properties = (
-                ('NEVRA', 'ElementName'),
-                ('Summary', 'Caption'))
-        if package_array:
-            package_generators = []
-            for pkg_spec in package_array:
-                exact_match = any(reg.match(pkg_spec) for reg in (
-                        software.RE_ENVRA, software.RE_NEVRA, software.RE_NA))
-                generator = (   i.to_instance()
-                            for i in software.find_package(ns,
-                                    pkg_spec=pkg_spec,
-                                    allow_duplicates=_allow_duplicates,
-                                    exact_match=exact_match))
-                if _available and not _repoid:
-                    # filter out installed packages
-                    # if repoid is given, there's no need to filter them out
-                    generator = (i for i in generator if i.InstallDate is None)
-                elif not _all and not _available:
-                    # filter out not installed packages
-                    generator = (  i for i in generator
-                                if i.InstallDate is not None)
-                # else: # _all is given - no need to filter
-                package_generators.append(generator)
-            instances = itertools.chain(*package_generators)
-
-        elif _all or _available:
-            instances = software.list_available_packages(ns,
-                    allow_installed=_all,
+    def execute(self, ns, package_array,
+            _allow_duplicates=False):
+        for pkg_spec in package_array:
+            for pkg in software.find_package(ns,
                     allow_duplicates=_allow_duplicates,
-                    repoid=_repoid)
-
-        else:
-            instances = software.list_installed_packages(ns)
-
-        return (properties, instances)
-
-class RepoLister(command.LmiInstanceLister):
-    DYNAMIC_PROPERTIES = True
-
-    def execute(self, ns, _all, _disabled):
-        if _all:
-            properties = (
-                    ('Repo id', 'Name'),
-                    ('Name', 'Caption'),
-                    ('Enabled', lambda i: i.EnabledState == 2))
-            enabled = None
-        else:
-            properties = (
-                    ('Repo id', 'Name'),
-                    ('Name', 'Description'))
-            enabled = not _disabled
-
-        return (properties, software.list_repositories(ns, enabled))
-
-class FileLister(command.LmiInstanceLister):
-    DYNAMIC_PROPERTIES = True
-
-    def verify_options(self, options):
-        file_types = { 'all', 'file', 'directory', 'symlink', 'fifo', 'device'}
-        if (   options['--type'] is not None
-           and options['--type'] not in file_types):
-            raise errors.LmiInvalidOptions(
-                    'invalid file type given, must be one of %s' % file_types)
-
-    def execute(self, ns, package, _type=None):
-        properties = [
-                ('Name'),
-                ('Type', lambda i: 
-                         software.FILE_TYPES[i.FileType]
-                    if   i.FileExists
-                    else 'Missing'),
-                ('FileSize', lambda i: i.FileSize),
-                ('Passed', lambda i: len(i.FailedFlags) < 1)]
-        if _type is not None:
-            del properties[1]
-
-        pkgs = list(software.find_package(ns, pkg_spec=package[0]))
-        if len(pkgs) < 1:
-            raise errors.LmiFailed(
-                    'no package matching "%s" found' % package[0])
-        if len(pkgs) > 1:
-            LOG().warn('more than one package found for "%s": %s',
-                    package[0], ', '.join(p.ElementName for p in pkgs))
-
-        return ( properties
-               , software.list_package_files(ns, pkgs[-1], file_type=_type))
-
-class Lister(command.LmiCommandMultiplexer):
-    """ List information about packages, repositories or files. """
-    COMMANDS = {
-            'pkgs' : PkgLister,
-            'repos' : RepoLister,
-            'files' : FileLister
-    }
+                    exact_match=False,
+                    pkg_spec=pkg_spec):
+                yield pkg
 
 class PkgInfo(command.LmiShowInstance):
     DYNAMIC_PROPERTIES = True
@@ -222,17 +127,19 @@ class PkgInfo(command.LmiShowInstance):
                 'Version',
                 'Release',
                 ('Summary', 'Caption'),
+                ('Installed', lambda i: 'no' if i.InstallDate is None
+                    else i.InstallDate.datetime.strftime('%a %b %d/%Y  %H:%M')),
                 'Description']
         pkgs = [   p.to_instance()
                for p in software.find_package(ns,
-                        pkg_spec=package[0],
+                        pkg_spec=package,
                         repoid=_repoid)]
         pkgs = [p for p in pkgs if not _installed or bool(p.InstallDate)]
         if len(pkgs) < 1:
-            raise errors.LmiFailed('no such package "%s" found' % package[0])
+            raise errors.LmiFailed('No such package "%s" found.' % package)
         if len(pkgs) > 1:
-            LOG().warn('more than one package found for "%s" : %s',
-                    package[0], ', '.join(p.ElementName for p in pkgs))
+            LOG().warn('More than one package found for "%s" : %s',
+                    package, ', '.join(p.ElementName for p in pkgs))
 
         return (properties, pkgs[-1])
 
@@ -246,31 +153,45 @@ class RepoInfo(command.LmiShowInstance):
             'Description')
 
     def transform_options(self, options):
-        options['repoid'] = options.pop('<repository>')[0]
+        options['repoid'] = options.pop('<repository>')
 
 class Show(command.LmiCommandMultiplexer):
-    """ Show information about packages or repositories. """
+    """
+    Show details of package or repository.
+
+    Usage:
+        %(cmd)s pkg [--installed | --repoid <repository>] <package>
+        %(cmd)s repo <repository>
+
+    Options:
+        --installed            Do not search available packages. This speeds up
+                               the operation when only installed packages shall
+                               be queried.
+        --repoid <repository>  Search just this repository.
+    """
     COMMANDS = { 'pkg' : PkgInfo, 'repo' : RepoInfo }
+    OWN_USAGE = True
 
 def for_each_package_specs(ns, pkg_specs, info, func,
         repoid=None, just_on_installed=True):
     """
     Iterate over package specification strings, find them on remote host,
     make them into ``LMI_SoftwareIdentity``, and pass them to given function.
-    
-    :param pkg_specs: (``list``) List of package specification strings.
-    :param info: (``str``) What is done with package. This is used in log
-        messages.
-    :param func: (``callable``) Any callable taking instance of
+
+    :param list pkg_specs: Package specification strings.
+    :param string info: What is done with package. This is used in log messages.
+    :param callable func: Any callable taking instance of
         ``LMI_SoftwareIdentity`` as the first and only argument.
-    :param repoid: (``str``) Optional repository id used in a search
+    :param string repoid: Optional repository id used in a search
         for corresponding software identity.
-    :param just_on_installed: (``bool``) Skip uninstalled software identities
+    :param boolean just_on_installed: Skip uninstalled software identities
         found.
-    :rtype: (``list``) List containing a subset of ``pkg_specs`` with items,
-        that were processed successfuly.
+    :returns: Pair with list containing a subset of ``pkg_specs`` with items,
+        that were processed successfuly and a list of errors for other packages.
+    :rtype: tuple
     """
     done_on = []
+    failed = []
     for pkg_spec in pkg_specs:
         if just_on_installed:
             identities = [
@@ -283,11 +204,15 @@ def for_each_package_specs(ns, pkg_specs, info, func,
             identities = list(software.find_package(ns,
                 pkg_spec=pkg_spec, repoid=repoid))
         if len(identities) < 1:
-            LOG().warn('failed to find any matching package for "%s",'
-                ' skipping', pkg_spec)
+            if just_on_installed:
+                msg = 'No such installed package "%s".' % pkg_spec
+            else:
+                msg = 'Failed to find package "%s".' % pkg_spec
+            LOG().warn(msg + ' Skipping.')
+            failed.append(msg)
             continue
         if len(identities) > 1:
-            LOG().warn('more than one package found for "%s": %s',
+            LOG().debug('More than one package found for "%s": %s',
                     pkg_spec,
                     ', '.join(software.get_package_nevra(i)
                         for i in identities))
@@ -295,8 +220,9 @@ def for_each_package_specs(ns, pkg_specs, info, func,
             func(identities[-1])
             done_on.append(pkg_spec)
         except errors.LmiFailed as err:
-            LOG().warn('failed to %s "%s": %s', info, pkg_spec, err)
-    return done_on
+            LOG().warn('Failed to %s "%s": %s', info, pkg_spec, err)
+            failed.append(err)
+    return done_on, failed
 
 class Install(command.LmiCheckResult):
     ARG_ARRAY_SUFFIX = '_array'
@@ -307,10 +233,13 @@ class Install(command.LmiCheckResult):
             option, this should contain 1 argument equal to --uri. Otherwise we
             expect the same list as ``<package_array>``.
         """
+        done_on, failed = result
         if options['--uri']:
-            return [options['--uri']] == result
-        if options['<package_array>'] != result:
-            return (False, ('failed to install packages: %s' %
+            return [options['--uri']] == done_on
+        if options['<package_array>'] != done_on:
+            if len(options['<package_array>']) == 1:
+                return (False, failed[0])
+            return (False, ('Failed to install packages: %s' %
                     ", ".join(set(options['<package_array>']) - set(result))))
         return True
 
@@ -322,9 +251,10 @@ class Install(command.LmiCheckResult):
         if _uri is not None:
             try:
                 software.install_from_uri(ns, _uri, force=_force)
-                return [_uri]
+                return ([_uri], [])
             except errors.LmiFailed as err:
-                LOG().warn('failed to install "%s": %s', _uri, err)
+                LOG().warn('Failed to install "%s": %s', _uri, err)
+                return ([], [err])
 
         else:
             return for_each_package_specs(ns, package_array, 'install',
@@ -344,8 +274,11 @@ class Update(command.LmiCheckResult):
             option, this should contain 1 argument equal to --uri. Otherwise we
             expect the same list as ``<package_array>``.
         """
-        if options['<package_array>'] != result:
-            return (False, ('failed to update packages: %s' %
+        done_on, failed = result
+        if options['<package_array>'] != done_on:
+            if len(options['<package_array>']) == 1:
+                return (False, failed[0])
+            return (False, ('Failed to update packages: %s' %
                     ", ".join(set(options['<package_array>']) - set(result))))
         return True
 
@@ -364,8 +297,11 @@ class Remove(command.LmiCheckResult):
     ARG_ARRAY_SUFFIX = '_array'
 
     def check_result(self, options, result):
-        if options['<package_array>'] != result:
-            return (False, ('failed to remove packages: %s' %
+        done_on, failed = result
+        if options['<package_array>'] != done_on:
+            if len(options['<package_array>']) == 1:
+                return (False, failed[0])
+            return (False, ('Failed to remove packages: %s' %
                     ", ".join(set(options['<package_array>']) - set(result))))
         return True
 
@@ -388,7 +324,7 @@ class Verify(command.LmiLister):
             if len(failed_checks):
                 failed_identity_checks.append((identity, failed_checks))
             else:
-                LOG().debug('package "%s" passed', identity.ElementName)
+                LOG().debug('Package "%s" passed.', identity.ElementName)
 
         for_each_package_specs(ns, package_array, 'verify', _verify_identity)
         for identity, checks in failed_identity_checks:
@@ -408,11 +344,11 @@ class ChangeEnabledState(command.LmiCheckResult):
     @property
     def enable(self):
         """ Whether to enable or disable repository. """
-        return True 
+        return True
 
     def check_result(self, options, result):
         if options['<repository_array>'] != result:
-            return (False, ('failed to %s repositories: %s' % (
+            return (False, ('Failed to %s repositories: %s' % (
                 'enable' if self.enable else 'disable',
                 ", ".join(set(options['<repository_array>']) - set(result)))))
         return True
@@ -437,6 +373,7 @@ class DisableRepository(ChangeEnabledState):
 Software = command.register_subcommands(
         'Software', __doc__,
         { 'list'    : Lister
+        , 'search'  : Search
         , 'show'    : Show
         , 'install' : Install
         , 'update'  : Update
